import { useNavigate } from "react-router-dom";
import { Form } from "../../Components/Form/Form";
import { notification } from 'antd';
import { useAppSelector } from "../../Store";
import { useAppDispatch } from "../../Store";
import { onChangeState, postQuotes } from "../../Store/slices/quotes.slice";
import { shallowEqual } from "react-redux";
import "./ContextFrom.css";

export const ContextForm = () => {
    const navigate = useNavigate()
    const dispatch = useAppDispatch()
    const {values} = useAppSelector(state => state.quotes, shallowEqual)

    const onChangeHandler = (e: React.ChangeEvent<HTMLInputElement>) => {
        const {name, value} = e.target
        dispatch(onChangeState({type: "input", name, value}))
    }

    const onChangeSelectHandler = (value: string) => {
        dispatch(onChangeState({type: "select", value})) 
    }

    const onSubmitHandler = async (e: React.FormEvent<HTMLFormElement>) => {
        e.preventDefault()

        const validate = Object.values(values).every(item => item !== "");

        if(validate) {
            dispatch(postQuotes(values))
            navigate(`/quotes/${values.category}`)
        } else {
            notification.warning({
                message: "Field all input",
                placement: "top"
            })
        }
    }

    return (
        <div className="ContextForm">
            <Form 
                onSubmit={onSubmitHandler} 
                onChangeSelect={onChangeSelectHandler} 
                onChange={onChangeHandler} 
                valueSelect={values.category}
                valueAuthor={values.author} 
                valueText={values.text}
            />
        </div>
    )
}