import { createSlice, createAsyncThunk } from "@reduxjs/toolkit";
import { TQuote, TQuoteState } from "../../interfaces/quote";
import { apiConnector } from "../../Integrations/api.connector";

export const getQuotes = createAsyncThunk(
    "quotes/getQuotes",
    async () => {
        try {
            return await apiConnector.getQuotes();
        } catch (e) {
            throw new Error(e as string)
        }
    }
)

export const postQuotes = createAsyncThunk(
    "quotes/postQuotes",
    async (body: TQuote) => {
        try {
            return await apiConnector.postQuotes(body);
        } catch (e) {
            throw new Error(e as string)
        }
    }
)

export const deleteQuote = createAsyncThunk(
    "quotes/deleteQuote",
    async (id: string, {dispatch}) => {
        try {
            dispatch(deleteLocalQuote(id))
            await apiConnector.deleteQuotes(id);
        } catch (e) {
            throw new Error(e as string)
        }
    }
)

const initialState: TQuoteState = {
    isLoading: false,
    quotes: null,
    values: {
        category: "",
        text: "",
        author: ""
    }
}

export const QuotesSlice = createSlice({
    name: "quotes",
    initialState,
    reducers: {
        deleteLocalQuote(state, action) {
            const index = state.quotes?.findIndex(item => item.id === action.payload) || 0
            state.quotes?.splice(index, 1)
        },
        onChangeState(state, action) {
            const {type, name, value} = action.payload as {type: string, name?: string, value: string}

            if(type === "input") {
                name === "text" && (state.values.text = value)
                name === "author" && (state.values.author = value)
            } else if(type === "select") {
                state.values.category = value
            }
        }
    },
    extraReducers: builder => {
        builder
        //Get
        .addCase(getQuotes.pending, (state) => {
            state.isLoading = true
        })
        .addCase(getQuotes.rejected, (state) => {
            state.isLoading = false
        })
        .addCase(getQuotes.fulfilled, (state, action) => {
            state.quotes = action.payload
        })

        //POST
        .addCase(postQuotes.pending, (state) => {
            state.isLoading = true
        })
        .addCase(postQuotes.rejected, (state) => {
            state.isLoading = false
        })
        .addCase(postQuotes.fulfilled, (state) => {
            state.isLoading = false;
        })

        // Delete
        .addCase(deleteQuote.pending, (state) => {
            state.isLoading = true
        })
        .addCase(deleteQuote.rejected, (state) => {
            state.isLoading = false
        })
        .addCase(deleteQuote.fulfilled, (state) => {
            state.isLoading = false
        })
    }
})

export const {deleteLocalQuote, onChangeState} = QuotesSlice.actions